﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class FrisbeeBehaviour : MonoBehaviour {
	
	GameManager gameMan;

    Vector3 viewPos;
    bool shouldCheck = true;
    public LayerMask cameraEdges;
	public Rigidbody rb;

	public float outTimerLimit;

	private float outTimer = 0;


	[Space]

	[Header ("Sound")]

	public AudioSource SFXSource;
	public AudioSource BGSSource;

	public AudioClip groundedSFX;
	public AudioClip impactSFX;

	private bool firstFrame = true;
	private bool grounded = false;

    void Start ()
    {
		rb.velocity = Vector3.zero;
		gameMan = Camera.main.GetComponent<GameManager>();
    }

    void Update()
	{
		/// Tweaked for sound

		float floatVelocity = (Mathf.Abs (rb.velocity.x) + Mathf.Abs (rb.velocity.y) + Mathf.Abs (rb.velocity.z));

		if (rb.velocity != Vector3.zero && floatVelocity > 0.1f) {
			transform.rotation = Quaternion.LookRotation (rb.velocity);
			grounded = false;
			if (!BGSSource.isPlaying) {
				BGSSource.Play ();
			}
			BGSSource.pitch = Mathf.Min(1, floatVelocity / 40);
			BGSSource.volume = Mathf.Min(1, floatVelocity / 40);
		}
		else if (!firstFrame && !grounded){
			if (transform.position.y < -1) {
				SFXSource.PlayOneShot (groundedSFX);
			} else {
				SFXSource.PlayOneShot (impactSFX);
			}
			BGSSource.Stop ();
			grounded = true;
		}
		firstFrame = false;
		/// End of

        viewPos = Camera.main.WorldToViewportPoint(transform.position);

        if (shouldCheck)
        {
            if (viewPos.x < 0 || viewPos.x > 1 || viewPos.y < 0 || viewPos.y > 1)
            {
                Debug.DrawLine(transform.position, -transform.forward * 500, Color.red);

                RaycastHit hit;
                if (Physics.Raycast(transform.position, -transform.forward, out hit, 500, cameraEdges))
                {
                    transform.position = new Vector3(hit.point.x, transform.position.y, hit.point.z);

                    StartCoroutine(WaitUntilInScreen());
                    shouldCheck = false;
                }
            }
        }
		if (viewPos.x < 0 || viewPos.x > 1 || viewPos.y < 0 || viewPos.y > 1) {
			outTimer += Time.deltaTime;
			if (outTimer > outTimerLimit) {
				outTimer = 0;
				gameMan.ResetRound ();
			}
		} else {
			outTimer = 0;
		}

    }

	void OnCollisionEnter(){
		SFXSource.PlayOneShot (impactSFX);
	}

    IEnumerator WaitUntilInScreen()
    {
        while (viewPos.x < 0 || viewPos.x > 1 || viewPos.y < 0 || viewPos.y > 1)
        {
            yield return null;
        }
        shouldCheck = true;
    }
}
