﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;
using XInputDotNetPure;

public class GameManager : MonoBehaviour {

    [Header("Teams")]

    public string teamOneName;
    public string teamTwoName;
    Dictionary<string, int> teams = new Dictionary<string, int>();
	public Rounds roundState; 
    public int scoreLimit;

    [Space]

    [Header("Players")]

    public PlayerController[] players;
    public Transform frisbee;
    public Transform goodSpawn;
    public Transform badSpawn;
	public int maxPlayers;

    [Space]

    [Header ("Texts")]

    public Text scoreText;
    public Text goalText;
    public Text setupText;
	public AnimationCurve scaleCurve;

	[Space]

	[Header ("Sound & Music")]

	public SoundMan soundMan;

	GamepadManager manager;

	[Space]

	[Header ("Startup")]

	public Vector3 cameraTopPosition;
	public Vector3 cameraTutorialPosition;
	public Vector3 cameraGamePosition;
	public float cameraLerpSpeed;
	public Text[] startTexts;
	public bool starting = false;
	public bool isTutoring = true;
    public float sandboxTime;
	public bool AIFill = false;
	public TextMesh AIFillNotice;

	private float startTextAlpha = 1f;
    private float startTimer = 0f;


    void Start()
    { 
        teams.Add(teamOneName, 0);
        teams.Add(teamTwoName, 0);
        manager = GamepadManager.Instance;
		transform.position = cameraTopPosition;
		scoreText.gameObject.SetActive (false);
		frisbee.gameObject.SetActive (false);
		isTutoring = true;

        for (int i = 0; i < maxPlayers ; i++)
		{
			PlayerIndex currentIndex = PlayerIndex.One;
			switch (i) {
			case 0:
				currentIndex = PlayerIndex.One;
				break;
			case 1:
				currentIndex = PlayerIndex.Two;
				break;
			case 2:
				currentIndex = PlayerIndex.Three;
				break;
			case 3:
				currentIndex = PlayerIndex.Four;
				break;
			}

			players [i].gameObject.SetActive (false);
        }

    }

    void Update()
	{

		if (Input.GetKeyDown("escape")) {
			Application.Quit ();
		}

		if (roundState == Rounds.Startup && !starting) {
			frisbee.gameObject.SetActive (false);
			for (int i = 1; i < manager.ConnectedTotal () + 1; i++) {
				if (manager.GetGamepad (i).GetButtonDown ("Start")) {
					starting = true;
					soundMan.PlaySFX ("catch");
					if (!soundMan.bgm.isPlaying) {
						soundMan.bgm.Play ();
					}
				}
			}
		} else if (roundState == Rounds.Startup) {
			for (int i = 1; i < manager.ConnectedTotal()  + 1; i++) {
				if (manager.GetGamepad (i).GetButtonDown ("Start")) {
					soundMan.PlaySFX ("catch");
					isTutoring = false;

					for (int j = 0; j < maxPlayers ; j++)
					{
						PlayerIndex currentIndex = PlayerIndex.One;
						switch (j) {
						case 0:
							currentIndex = PlayerIndex.One;
							break;
						case 1:
							currentIndex = PlayerIndex.Two;
							break;
						case 2:
							currentIndex = PlayerIndex.Three;
							break;
						case 3:
							currentIndex = PlayerIndex.Four;
							break;
						}

						players[j].gameObject.SetActive(GamePad.GetState(currentIndex).IsConnected);
						
						if (AIFill && !GamePad.GetState(currentIndex).IsConnected) {
							players[j].gameObject.SetActive(true);
							players [j].AIControlled = true;
						}
					}
				
				}
				if (manager.GetGamepad (i).GetButtonDown ("Back")) {
					AIFill = !AIFill;
					soundMan.PlaySFX ("throw");
					if (AIFill) {
						AIFillNotice.text = "AI fill is enabled";
					} else {
						AIFillNotice.text = "AI fill is disabled";
					}
				}
			}
		}
        if (roundState == Rounds.Win)
        {
			for (int i = 1; i < manager.ConnectedTotal(); i++)
            {
				if (manager.GetGamepad(i).GetButtonDown("Start"))
				{
					ResetGame ();
                }
            }
        }
    }

	void FixedUpdate(){


		if (roundState == Rounds.Startup) {
			if (starting) {
				foreach (Text startText in startTexts) {
					startText.color = new Color (startText.color.r, startText.color.g, startText.color.b, startTextAlpha);
					startTextAlpha = Mathf.Clamp (startTextAlpha - Time.deltaTime, 0, 1);
				}
				if (isTutoring) {
					transform.position = Vector3.Lerp (transform.position, cameraTutorialPosition, cameraLerpSpeed);
					//startText.color = new Color (startText.color.r, startText.color.g, startText.color.b, startTextAlpha);
					//startTextAlpha = Mathf.Clamp (startTextAlpha - Time.deltaTime, 0, 1);


				} else {

					transform.position = Vector3.Lerp (transform.position, cameraGamePosition, cameraLerpSpeed);
					if (Mathf.Floor (transform.position.x) == Mathf.Floor (cameraGamePosition.x) &&
					   Mathf.Floor (transform.position.y) == Mathf.Floor (cameraGamePosition.y) &&
					   Mathf.Floor (transform.position.z) == Mathf.Floor (cameraGamePosition.z)) {
						scoreText.gameObject.SetActive (true);
						UpdateUI ();
						startTimer += Time.deltaTime;
						if (startTimer > sandboxTime) {
							starting = false;
							foreach (Text startText in startTexts) {
								startText.color = new Color (startText.color.r, startText.color.g, startText.color.b, 1f);
								startText.gameObject.SetActive (false);
							}
							frisbee.gameObject.SetActive (true);
							startTimer = 0f;

							ResetRound ();
						}
					}

				}
			}
		}
	}

    public void Score(string _teamName)
    {
        teams[_teamName] += 1;
        if (teams[_teamName] == scoreLimit)
            StartCoroutine(WinFeedback(_teamName));
        else
            StartCoroutine(GoalFeedback(_teamName));

        UpdateUI();
    }

	void UpdateUI()
    {
		if (roundState == Rounds.Startup) {
			scoreText.text = "warming up";
		}
		else{
			string[] teamNames = new string[2];

			if (teams [teamOneName] >= scoreLimit - 1) {
				teamNames [0] = "<color=#FF9A9AFF>" + teams [teamOneName].ToString () + "</color>";
			} else {
				teamNames [0] = teams [teamOneName].ToString ();
			}
			if (teams [teamTwoName] >= scoreLimit - 1) {
				teamNames [1] = "<color=#FF9A9AFF>" + teams [teamTwoName].ToString () + "</color>";
			} else {
				teamNames [1] = teams [teamTwoName].ToString ();
			}

			scoreText.text = teamNames [1] + " - " + teamNames [0];
		}
    }

    IEnumerator SetupCoroutine()
    {
        roundState = Rounds.Setup;

        yield return new WaitForSeconds(.5f);

        setupText.gameObject.SetActive(true);
        setupText.rectTransform.localScale = new Vector3(0, 0, 1);
        setupText.text = "READY?";

        float scaleTime = 0f;
        while (setupText.rectTransform.localScale != new Vector3(1, 1, 1))
        {
            float scale = scaleCurve.Evaluate(scaleTime);
            setupText.rectTransform.localScale = new Vector3(scale, scale, 1);
            scaleTime += Time.deltaTime;
            yield return null;
        }

        yield return new WaitForSeconds(1);

        setupText.rectTransform.localScale = new Vector3(0, 0, 1);
        setupText.text = "GO!";

        scaleTime = 0f;
        while (setupText.rectTransform.localScale != new Vector3(1, 1, 1))
        {
            float scale = scaleCurve.Evaluate(scaleTime);
            setupText.rectTransform.localScale = new Vector3(scale, scale, 1);
            scaleTime += Time.deltaTime;
            yield return null;
		}

		/// SOUND
		soundMan.PlaySFX ("startRound");
		/// ENDOF

        roundState = Rounds.Action;

        yield return new WaitForSeconds(.5f);
        setupText.gameObject.SetActive(false);
    }

    IEnumerator GoalFeedback(string _teamName)
    {
        roundState = Rounds.Score;
        yield return new WaitForSeconds(1);

        goalText.gameObject.SetActive(true);
        goalText.rectTransform.localScale = new Vector3 (0, 0, 1);
		goalText.text = _teamName + " SCORE !";

        float scaleTime = 0f;
        while (goalText.rectTransform.localScale != new Vector3(1, 1, 1))
        {
            float scale = scaleCurve.Evaluate(scaleTime);
            goalText.rectTransform.localScale = new Vector3(scale, scale, 1);
            scaleTime += Time.deltaTime;
            yield return null;
        }

		soundMan.PlaySFX ("endRound");
        yield return new WaitForSeconds(2f);
        goalText.gameObject.SetActive(false);

		ResetRound (_teamName);
    }

    IEnumerator WinFeedback(string _teamName)
    {
        roundState = Rounds.Win;
        yield return new WaitForSeconds(1);
        goalText.gameObject.SetActive(true);
        goalText.rectTransform.localScale = new Vector3(0, 0, 1);
        goalText.text = _teamName + " WIN !";
		soundMan.PlaySFX ("win");
        float scaleTime = 0f;
        while (goalText.rectTransform.localScale != new Vector3(1, 1, 1))
        {
            float scale = scaleCurve.Evaluate(scaleTime);
            goalText.rectTransform.localScale = new Vector3(scale, scale, 1);
            scaleTime += Time.deltaTime;
            yield return null;
        }
    }

	public void ResetRound(string _teamName=null){

		foreach (PlayerController player in players)
		{
			player.transform.position = player.originPos;
			if (player.fakeFrisbee.activeSelf)
			{
				player.fakeFrisbee.SetActive(false);
				player.realFrisbee.useGravity = true;
				player.moveSpeed = player.originSpeed;
			}
		}

		frisbee.gameObject.SetActive(true);
		frisbee.parent = null;
		frisbee.gameObject.GetComponent<Rigidbody> ().velocity = new Vector3();
		if (_teamName == teamOneName)
			frisbee.position = goodSpawn.position;
		else if (_teamName == teamTwoName) {
			frisbee.position = badSpawn.position;
		} else {
			frisbee.position = new Vector3(-11.5f, 0f, 0f);
		}

		StartCoroutine(SetupCoroutine());
		UpdateUI ();
	}

	public void ResetGame(){
		goalText.gameObject.SetActive(false);
		teams [teamTwoName] = 0;
		teams [teamOneName] = 0;
		ResetRound ();
		UpdateUI ();
	}
}
