﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class PlayerController : MonoBehaviour {

    GameManager gameMan;

    [Header("Player")]

    public x360_Gamepad gamepad;
    GamepadManager manager;
    public int player;
    public string teamName;
    public Vector3 originPos;
    public Animator anim;

    [Space]

    [Header("Movement")]

    public Rigidbody rb;
    public float moveSpeed;
    public float originSpeed;
    float hinput;
    float vinput;
    Vector3 direction;
    Vector3 movement;
    public ParticleSystem dust;
    public ParticleSystem grass;

    [Space]

    [Header("Frisbee")]

    public GameObject fakeFrisbee;
    public Rigidbody realFrisbee;
    public float frisbeeSpeed;
    public LayerMask playerLayer;
    public LayerMask frisbeeLayer;
    public GameObject frisbeeHitbox;
    Vector3 frisbeeBumpDirection;
    public float frisbeeBumpForce;

    [Space]

    [Header ("Dash")]

    bool isDashing;
    bool canDash;
    public float dashTime;
    public float dashSpeed;
    public float dashRecovery;
    public float dashCooldown;
    public GameObject dashHitbox;
    public float stunTime;
    public float stunSpeed;

	public Text catchText;

	[Space]

	[Header ("Sound")]

	public AudioSource SFXSource;

	public AudioClip[] pawStepSFX;
	public AudioClip dashSFX;
	public AudioClip kaiSFX;
	public AudioClip percussionSFX;
	public AudioClip catchSFX;
	public AudioClip throwSFX;

	private int footStepCounter = 0;
	private float footStepEvery = 0.4f; // footstep every 0.2 second
	private float currentFootstepStep = 0f;

	[Space]

	[Header ("AI")]

	public bool AIControlled;
	public GameObject[] goals;

	private Vector3 AIMoveObjective;
	private bool AINothingToDo = true;
	private float AIDashWait = 0;
	private float AIGiveWait = 0;
	private bool AIIntelligent = false;

    void Start()
	{
		AIMoveObjective = transform.position;
        originPos = transform.position;
        manager = GamepadManager.Instance;
        gamepad = manager.GetGamepad(player);
        canDash = true;
        gameMan = Camera.main.GetComponent<GameManager>();
        originSpeed = moveSpeed;
    }

	void Update ()
	{
		if (!isDashing)
			SetDirection ();
		
		/// AI CONTROL
		if (AIControlled) {
			foreach (PlayerController player in gameMan.players) {
				if (player.gameObject != gameObject && player.teamName == teamName && (!player.AIIntelligent || !player.AIControlled)){
					AIIntelligent = true;
				}
			}
			if (!isDashing) {
				if (gameMan.roundState != Rounds.Action) {
					if (AINothingToDo) {
						AIMoveObjective = new Vector3 (Random.value * (-20f), transform.position.y, Random.value * 40f - 25f);
						AINothingToDo = false;
					} else {
						if (Vector3.Distance (AIMoveObjective, transform.position) < 5f) {
							AINothingToDo = true;
						}
					}
				} 
				else if (gameMan.roundState == Rounds.Action) {
					
					if (fakeFrisbee.activeSelf) {	//Has frisbee
						GameObject objectiveGoal = null;
						foreach (GameObject thisGoal in goals) {
							if (thisGoal.GetComponent<GoalBehaviour> ().goalOwner == teamName) {
								objectiveGoal = thisGoal;
								break;
							}
						}
						if (objectiveGoal != null) {
							AIMoveObjective = objectiveGoal.transform.position;

							foreach (PlayerController player in gameMan.players) {
								if (player.teamName != teamName && 
									Vector3.Distance(player.gameObject.transform.position, transform.position) < 6f) {
									AIGiveWait += Time.deltaTime + Time.deltaTime * Random.value * 0.3f;
									if (AIGiveWait > 1) {
										AIGiveWait = 0;
										ThrowFrisbee ();
									}
									break;
								}
							}
						}

					} else {	//Does not have frisbee
						AIGiveWait = 0;
						if (AIIntelligent){
							if (realFrisbee.transform.parent == null) {
								AIMoveObjective = realFrisbee.transform.position;
							} else {
								PlayerController holder = realFrisbee.gameObject.GetComponentInParent<PlayerController> ();
								if (holder.teamName == teamName) {
									GameObject objectiveGoal = null;
									foreach (GameObject thisGoal in goals) {
										if (thisGoal.GetComponent<GoalBehaviour> ().goalOwner == teamName) {
											objectiveGoal = thisGoal;
											break;
										}
									}
									if (objectiveGoal != null) {
										AIMoveObjective = objectiveGoal.transform.position;
									}
								} else {
									AIMoveObjective = holder.transform.position;
									if (Vector3.Distance (AIMoveObjective, transform.position) < 10f) {
										AIDashWait += Time.deltaTime * Random.value + Time.deltaTime;
										if (AIDashWait > 1) {
											Dash ();
											AIDashWait = 0f;
										}
									} else {
										AIDashWait = 0f;
									}
								}
							}
						}
						else{
							if (Vector3.Distance (AIMoveObjective, transform.position) < 5f){
								AIMoveObjective = new Vector3 (Random.value * (-20f), transform.position.y, Random.value * 40f - 25f);
							}
						}
					}
				}
				Vector3 objDirection = AIMoveObjective - transform.position;
				if (Vector3.Distance (AIMoveObjective, transform.position) > 1f) {
					hinput = objDirection.normalized.x;
					vinput = objDirection.normalized.z;
				} else {
					hinput = 0f;
					vinput = 0f;
				}
			} 
		} 
		/// END OF

		else {

			if (gamepad.GetButtonDown ("A") && (gameMan.roundState == Rounds.Action || gameMan.roundState == Rounds.Startup)) {
				if (fakeFrisbee.activeSelf) {
					ThrowFrisbee ();
				} else {
					Dash ();
				}
			}
		}
	}

    void FixedUpdate()
    {
        if (!isDashing)
        {
            if (gameMan.roundState == Rounds.Setup)
            {
                rb.velocity = Vector3.zero;
            }

            else
            {
                movement = direction * moveSpeed;
                rb.velocity = new Vector3(movement.x, rb.velocity.y, movement.z);

				/// FOOTSTEP SOUND
				if (movement.x != 0f || movement.z != 0f) {
					
					float moveExtent = Mathf.Abs (hinput) + Mathf.Abs (vinput);
					moveExtent = moveSpeed / originSpeed;
					moveExtent = 1f;

					currentFootstepStep += Time.deltaTime*moveExtent;
					if (currentFootstepStep > footStepEvery) {
						currentFootstepStep = 0f;
						PlayFootstepSFX ();
					}
				}
				/// END OF
            }

            if (rb.velocity != Vector3.zero)
            {
                if (!anim.GetBool("isMoving"))
                    anim.SetBool("isMoving", true);

                if (!grass.isPlaying)
                    grass.Play();
                if (!dust.isPlaying)
                    dust.Play();
            }

            else
            {
                if (anim.GetBool("isMoving"))
                    anim.SetBool("isMoving", false);

                if (grass.isPlaying)
                    grass.Stop();
                if (dust.isPlaying)
                    dust.Stop();
            }
        }
    }

    void SetDirection()
    {
		if (!AIControlled) {
			hinput = gamepad.GetStick_L ().X;
			vinput = gamepad.GetStick_L ().Y;
		}

        direction = new Vector3(hinput, 0, vinput).normalized;

        if (direction != Vector3.zero)
        {
            transform.rotation = Quaternion.LookRotation(direction);
        }
    }

    void ThrowFrisbee()
	{
		SFXSource.PlayOneShot (throwSFX);
        realFrisbee.gameObject.SetActive(true);
        realFrisbee.transform.parent = null;
        realFrisbee.velocity = transform.forward * frisbeeSpeed;
        fakeFrisbee.SetActive(false);

        StartCoroutine(DelayTriggerActivation(.3f));
        dashHitbox.SetActive(false);
        //print("thrown");

    }

    void Dash()
    {
        if (canDash)
        {
			/// AUDIO
			SFXSource.PlayOneShot (dashSFX);
			/// END OF
		
            StartCoroutine(DashCoroutine());
        }
    }
    
    void OnTriggerEnter(Collider col)
    {
        if (col.tag == "Frisbee")
        {
            if (!fakeFrisbee.activeSelf)
            {
                fakeFrisbee.SetActive(true);
                col.transform.SetParent(transform);
                realFrisbee.velocity = Vector3.zero;
                col.transform.localPosition = new Vector3 (0, 2.3f, 1);
                col.transform.localRotation = Quaternion.identity;
                Physics.IgnoreLayerCollision(9, 10, true); //frisbee et bounds

                if (realFrisbee.useGravity)
                    realFrisbee.useGravity = false;

                col.gameObject.SetActive(false);

                frisbeeHitbox.SetActive(false);
                dashHitbox.SetActive(true);

                anim.SetTrigger("grabs");
				SFXSource.PlayOneShot (catchSFX);
                StartCoroutine(CatchFeedback());
            }
        }

        if (col.tag == "Player")
        {
            if (fakeFrisbee.activeSelf)
            {
                PlayerController player = col.GetComponentInParent<PlayerController>();

                if (player != null && player.isDashing && player.teamName != teamName)
                { 
                    /// SOUND
					SFXSource.PlayOneShot(kaiSFX);
					SFXSource.PlayOneShot(percussionSFX);
					/// END OF

					realFrisbee.gameObject.SetActive(true);
                    realFrisbee.transform.parent = null;
                    realFrisbee.useGravity = true;
                    realFrisbee.velocity = Vector3.zero;
                    frisbeeBumpDirection = col.transform.forward + new Vector3(Random.Range(-0.5f, 0.5f), 1, 0);
                    realFrisbee.AddForce(frisbeeBumpDirection * frisbeeBumpForce, ForceMode.Impulse);
                    Physics.IgnoreLayerCollision(9, 10, false);

                    fakeFrisbee.SetActive(false);
                    dashHitbox.SetActive(false);

                    StartCoroutine(DelayTriggerActivation(1));
                    player.frisbeeHitbox.SetActive(false);
                    StartCoroutine(player.DelayTriggerActivation(1));

                    StartCoroutine(StunTimer());
                    anim.SetTrigger("stunned");
                }
            }
        }
    }

    public IEnumerator DelayTriggerActivation(float _delay)
    {
        yield return new WaitForSeconds(_delay);
        frisbeeHitbox.SetActive(true);
    }

    IEnumerator DashCoroutine()
    {
        //Vector3 startPos = transform.position;
        isDashing = true;
        canDash = false;
        anim.SetTrigger("dashes");
        rb.velocity = transform.forward * dashSpeed;
        Physics.IgnoreLayerCollision(8, 8, true); //player layer
      //  rb.AddForce(transform.forward * dashSpeed, ForceMode.VelocityChange);
     //   yield return new WaitForFixedUpdate();
        yield return new WaitForSeconds(dashTime);

        rb.velocity = Vector3.zero;
        Physics.IgnoreLayerCollision(8, 8, false); //player layer
        //print((startPos - transform.position).magnitude);
        yield return new WaitForSeconds(dashRecovery);

        isDashing = false;
        yield return new WaitForSeconds(dashCooldown);

        canDash = true;
    }

    IEnumerator StunTimer()
    {
        moveSpeed = stunSpeed;
        canDash = false;
        yield return new WaitForSeconds(stunTime);
        moveSpeed = originSpeed;
        canDash = true;
    }

    IEnumerator CatchFeedback()
    {
        catchText.gameObject.SetActive(true);
        catchText.rectTransform.localScale = Vector3.zero;

        float scale = 0f;
        while (catchText.rectTransform.localScale.x < 1)
        {
            scale += Time.deltaTime * 10;
            catchText.rectTransform.localScale = new Vector3(scale, scale, scale);
            yield return null;
        }

        yield return new WaitForSeconds(.5f);

        while (catchText.rectTransform.localScale.x > 0)
        {
            scale -= Time.deltaTime * 20;
            catchText.rectTransform.localScale = new Vector3(scale, scale, scale);
            yield return null;
        }

        catchText.gameObject.SetActive(false);
    }

	/// 
	/// SOUND
	/// 

	void PlayFootstepSFX(){
		SFXSource.PlayOneShot (pawStepSFX [footStepCounter]);
		footStepCounter = (footStepCounter + 1) % pawStepSFX.Length;
	}

	/// ENDOF
}
